@@ -32,7 +32,11 @@ pub fn check_version(version: &str, path: &str) -> Result<()> {
     let (rmajor, rminor, rbugfix, rbeta) = parse_version(VERSION)?;
     let (smajor, sminor, sbugfix, sbeta) = parse_version(version)?;
 
-    if rmajor == smajor && rminor == sminor && rbugfix >= sbugfix && rbeta == sbeta {
+    if rmajor == smajor
+        && rminor == sminor
+        && rbugfix >= sbugfix
+        && rbeta == sbeta
+    {
         Ok(())
     } else {
         bail!(
@@ -61,7 +65,10 @@ fn resolve_fn_recursive<'lua, 'a>(
 }
 
 /// This function resolves paths like `builtin.func_foo`, `custom.func_bar` into lua functions.
-pub fn resolve_fn<'lua>(globals: &mlua::Table<'lua>, path: &str) -> Result<mlua::Function<'lua>> {
+pub fn resolve_fn<'lua>(
+    globals: &mlua::Table<'lua>,
+    path: &str,
+) -> Result<mlua::Function<'lua>> {
     let path = format!("xplr.fn.{}", path);
     resolve_fn_recursive(globals, path.split('.'))
 }
@@ -98,10 +105,11 @@ pub fn extend(lua: &Lua, path: &str) -> Result<Config> {
 
     lua.load(&script).set_name("init")?.exec()?;
 
-    let version: String = match globals.get("version").and_then(|v| lua.from_value(v)) {
-        Ok(v) => v,
-        Err(_) => bail!("'version' must be defined globally in {}", path),
-    };
+    let version: String =
+        match globals.get("version").and_then(|v| lua.from_value(v)) {
+            Ok(v) => v,
+            Err(_) => bail!("'version' must be defined globally in {}", path),
+        };
 
     check_version(&version, path)?;
 
@@ -124,7 +132,7 @@ pub fn call<'lua, R: Deserialize<'lua>>(
 }
 
 #[cfg(test)]
-mod test {
+mod tests {
 
     use super::*;
 
@@ -133,30 +141,24 @@ mod test {
         assert!(check_version(VERSION, "foo path").is_ok());
 
         // Current release if OK
-        assert!(check_version("0.15.2", "foo path").is_ok());
+        assert!(check_version("0.16.3", "foo path").is_ok());
 
         // Prev major release is ERR
         // - Not yet
 
         // Prev minor release is ERR (Change when we get to v1)
-        assert!(check_version("0.14.2", "foo path").is_err());
+        assert!(check_version("0.15.3", "foo path").is_err());
 
         // Prev bugfix release is OK
-        assert!(check_version("0.15.1", "foo path").is_ok());
+        assert!(check_version("0.16.2", "foo path").is_ok());
 
         // Next major release is ERR
-        assert!(check_version("1.15.2", "foo path").is_err());
+        assert!(check_version("1.16.3", "foo path").is_err());
 
         // Next minor release is ERR
-        assert!(check_version("0.16.2", "foo path").is_err());
+        assert!(check_version("0.17.3", "foo path").is_err());
 
         // Next bugfix release is ERR (Change when we get to v1)
-        assert!(check_version("0.15.3", "foo path").is_err());
-    }
-
-    #[test]
-    fn test_upgrade_guide_has_latest_version() {
-        let guide = include_str!("../docs/en/src/upgrade-guide.md");
-        assert!(guide.contains(VERSION));
+        assert!(check_version("0.16.4", "foo path").is_err());
     }
 }
