@@ -1,6 +1,7 @@
 #![allow(clippy::too_many_arguments)]
 
 use crate::app;
+use crate::app::ExternalMsg;
 use crate::cli::Cli;
 use crate::event_reader::EventReader;
 use crate::explorer;
@@ -8,13 +9,12 @@ use crate::lua;
 use crate::pipe_reader;
 use crate::pwd_watcher;
 use crate::ui;
-use anyhow::Result;
+use anyhow::{bail, Error, Result};
 use crossterm::event;
 use crossterm::execute;
 use crossterm::terminal as term;
 use mlua::LuaSerdeExt;
 use std::fs;
-use std::io;
 use std::io::Write;
 use std::path::PathBuf;
 use std::process::{Command, ExitStatus, Stdio};
@@ -22,11 +22,14 @@ use std::sync::mpsc;
 use tui::backend::CrosstermBackend;
 use tui::Terminal;
 
-pub fn get_tty() -> io::Result<fs::File> {
-    fs::OpenOptions::new()
-        .read(true)
-        .write(true)
-        .open("/dev/tty")
+pub fn get_tty() -> Result<fs::File> {
+    let tty = "/dev/tty";
+    match fs::OpenOptions::new().read(true).write(true).open(&tty) {
+        Ok(f) => Ok(f),
+        Err(e) => {
+            bail!(format!("Failed to open {}. {}", tty, e))
+        }
+    }
 }
 
 fn call_lua(
@@ -47,7 +50,7 @@ fn call_lua(
     lua::call(lua, func, arg)
 }
 
-fn call(app: &app::App, cmd: app::Command, silent: bool) -> io::Result<ExitStatus> {
+fn call(app: &app::App, cmd: app::Command, silent: bool) -> Result<ExitStatus> {
     let focus_index = app
         .directory_buffer
         .as_ref()
@@ -66,7 +69,10 @@ fn call(app: &app::App, cmd: app::Command, silent: bool) -> io::Result<ExitStatu
         .env("XPLR_PID", &app.pid.to_string())
         .env(
             "XPLR_INPUT_BUFFER",
-            app.input_buffer.clone().unwrap_or_default(),
+            app.input
+                .as_ref()
+                .map(|i| i.value().to_string())
+                .unwrap_or_default(),
         )
         .env("XPLR_FOCUS_PATH", app.focused_node_str())
         .env("XPLR_FOCUS_INDEX", focus_index)
@@ -90,6 +96,7 @@ fn call(app: &app::App, cmd: app::Command, silent: bool) -> io::Result<ExitStatu
         .stderr(stderr)
         .args(cmd.args)
         .status()
+        .map_err(Error::new)
 }
 
 fn start_fifo(path: &str, focus_path: &str) -> Result<fs::File> {
@@ -109,6 +116,23 @@ pub struct Runner {
     extra_config_files: Vec<PathBuf>,
     on_load: Vec<app::ExternalMsg>,
     read_only: bool,
+    selection: Vec<PathBuf>,
+}
+
+// In unix system, the std::env::current_dir() calls libc getcwd() that
+// returns physical path. As a workaround, this function tries to use `PWD`
+// environment variable that is configured by shell.
+fn get_current_dir() -> Result<PathBuf, std::io::Error> {
+    let cur = std::env::current_dir();
+    if let Ok(pwd) = std::env::var("PWD") {
+        if pwd.is_empty() {
+            cur
+        } else {
+            Ok(PathBuf::from(pwd))
+        }
+    } else {
+        cur
+    }
 }
 
 impl Runner {
@@ -119,14 +143,19 @@ impl Runner {
 
     /// Create a new runner object passing the given arguments
     pub fn from_cli(cli: Cli) -> Result<Self> {
-        let basedir = std::env::current_dir()?;
-        let mut pwd = cli
-            .path
-            .map(|p| if p.is_relative() { basedir.join(p) } else { p })
-            .unwrap_or_else(|| basedir.clone());
+        let basedir = get_current_dir()?;
+        let basedir_clone = basedir.clone();
+        let mut paths = cli.paths.into_iter().map(|p| {
+            if p.is_relative() {
+                basedir_clone.join(p)
+            } else {
+                p
+            }
+        });
+        let mut pwd = paths.next().unwrap_or_else(|| basedir.clone());
         let mut focused_path = None;
 
-        if pwd.is_file() {
+        if cli.force_focus || pwd.is_file() {
             focused_path = pwd.file_name().map(|p| p.into());
             pwd = pwd.parent().map(|p| p.into()).unwrap_or(basedir);
         }
@@ -138,6 +167,7 @@ impl Runner {
             extra_config_files: cli.extra_config,
             on_load: cli.on_load,
             read_only: cli.read_only,
+            selection: paths.collect(),
         })
     }
 
@@ -145,7 +175,12 @@ impl Runner {
     pub fn run(self) -> Result<Option<String>> {
         // Why unsafe? See https://github.com/sayanarijit/xplr/issues/309
         let lua = unsafe { mlua::Lua::unsafe_new() };
-        let mut app = app::App::create(self.pwd, &lua, self.config_file, self.extra_config_files)?;
+        let mut app = app::App::create(
+            self.pwd,
+            &lua,
+            self.config_file,
+            self.extra_config_files,
+        )?;
         app.config.general.read_only = self.read_only;
 
         fs::create_dir_all(app.session_path.clone())?;
@@ -155,6 +190,10 @@ impl Runner {
 
         app = app.explore_pwd()?;
 
+        for file in self.selection {
+            app = app.select_path(file.to_string_lossy().to_string())?;
+        }
+
         app = if let Some(f) = self
             .focused_path
             .clone()
@@ -185,19 +224,19 @@ impl Runner {
         // let mut stdout = stdout.lock();
         execute!(stdout, term::EnterAlternateScreen)?;
 
-        let mut fifo: Option<fs::File> = if let Some(path) = app.config.general.start_fifo.as_ref()
-        {
-            // TODO remove duplicate segment
-            match start_fifo(path, &app.focused_node_str()) {
-                Ok(file) => Some(file),
-                Err(e) => {
-                    app = app.log_error(e.to_string())?;
-                    None
+        let mut fifo: Option<fs::File> =
+            if let Some(path) = app.config.general.start_fifo.as_ref() {
+                // TODO remove duplicate segment
+                match start_fifo(path, &app.focused_node_str()) {
+                    Ok(file) => Some(file),
+                    Err(e) => {
+                        app = app.log_error(e.to_string())?;
+                        None
+                    }
                 }
-            }
-        } else {
-            None
-        };
+            } else {
+                None
+            };
 
         let mut last_focus: Option<app::Node> = None;
 
@@ -213,7 +252,11 @@ impl Runner {
         terminal.hide_cursor()?;
 
         // Threads
-        pwd_watcher::keep_watching(app.pwd.as_ref(), tx_msg_in.clone(), rx_pwd_watcher)?;
+        pwd_watcher::keep_watching(
+            app.pwd.as_ref(),
+            tx_msg_in.clone(),
+            rx_pwd_watcher,
+        )?;
         let mut event_reader = EventReader::new(tx_msg_in.clone());
         event_reader.start();
 
@@ -231,6 +274,8 @@ impl Runner {
                         match msg {
                             // NOTE: Do not schedule critical tasks via tx_msg_in in this loop.
                             // Try handling them immediately.
+                            //
+                            // TODO: Remove boilerplate code.
                             Enque(task) => {
                                 tx_msg_in.send(task)?;
                             }
@@ -280,8 +325,13 @@ impl Runner {
                                 explorer::explore_async(
                                     app.explorer_config.clone(),
                                     app.pwd.clone().into(),
-                                    app.focused_node().map(|n| n.relative_path.clone().into()),
-                                    app.directory_buffer.as_ref().map(|d| d.focus).unwrap_or(0),
+                                    app.focused_node().map(|n| {
+                                        n.relative_path.clone().into()
+                                    }),
+                                    app.directory_buffer
+                                        .as_ref()
+                                        .map(|d| d.focus)
+                                        .unwrap_or(0),
                                     tx_msg_in.clone(),
                                 );
                                 tx_pwd_watcher.send(app.pwd.clone())?;
@@ -291,8 +341,13 @@ impl Runner {
                                 explorer::explore_recursive_async(
                                     app.explorer_config.clone(),
                                     app.pwd.clone().into(),
-                                    app.focused_node().map(|n| n.relative_path.clone().into()),
-                                    app.directory_buffer.as_ref().map(|d| d.focus).unwrap_or(0),
+                                    app.focused_node().map(|n| {
+                                        n.relative_path.clone().into()
+                                    }),
+                                    app.directory_buffer
+                                        .as_ref()
+                                        .map(|d| d.focus)
+                                        .unwrap_or(0),
                                     tx_msg_in.clone(),
                                 );
                                 tx_pwd_watcher.send(app.pwd.clone())?;
@@ -307,7 +362,11 @@ impl Runner {
                                 let focus = app.focused_node();
                                 if focus != last_focus.as_ref() {
                                     if let Some(ref mut file) = fifo {
-                                        writeln!(file, "{}", app.focused_node_str())?;
+                                        writeln!(
+                                            file,
+                                            "{}",
+                                            app.focused_node_str()
+                                        )?;
                                     };
                                     last_focus = focus.cloned();
                                 }
@@ -323,7 +382,8 @@ impl Runner {
                                             mouse_enabled = true;
                                         }
                                         Err(e) => {
-                                            app = app.log_error(e.to_string())?;
+                                            app =
+                                                app.log_error(e.to_string())?;
                                         }
                                     }
                                 }
@@ -335,8 +395,10 @@ impl Runner {
                                 } else {
                                     app::ExternalMsg::EnableMouse
                                 };
-                                app = app
-                                    .handle_task(app::Task::new(app::MsgIn::External(msg), None))?;
+                                app = app.handle_task(app::Task::new(
+                                    app::MsgIn::External(msg),
+                                    None,
+                                ))?;
                             }
 
                             DisableMouse => {
@@ -349,14 +411,18 @@ impl Runner {
                                             mouse_enabled = false;
                                         }
                                         Err(e) => {
-                                            app = app.log_error(e.to_string())?;
+                                            app =
+                                                app.log_error(e.to_string())?;
                                         }
                                     }
                                 }
                             }
 
                             StartFifo(path) => {
-                                fifo = match start_fifo(&path, &app.focused_node_str()) {
+                                fifo = match start_fifo(
+                                    &path,
+                                    &app.focused_node_str(),
+                                ) {
                                     Ok(file) => Some(file),
                                     Err(e) => {
                                         app = app.log_error(e.to_string())?;
@@ -377,10 +443,14 @@ impl Runner {
                                     fifo = None;
                                     std::mem::drop(file);
                                 } else {
-                                    fifo = match start_fifo(&path, &app.focused_node_str()) {
+                                    fifo = match start_fifo(
+                                        &path,
+                                        &app.focused_node_str(),
+                                    ) {
                                         Ok(file) => Some(file),
                                         Err(e) => {
-                                            app = app.log_error(e.to_string())?;
+                                            app =
+                                                app.log_error(e.to_string())?;
                                             None
                                         }
                                     }
@@ -391,10 +461,12 @@ impl Runner {
                                 match call_lua(&app, &lua, &func, false) {
                                     Ok(Some(msgs)) => {
                                         for msg in msgs {
-                                            app = app.handle_task(app::Task::new(
-                                                app::MsgIn::External(msg),
-                                                None,
-                                            ))?;
+                                            app = app.handle_task(
+                                                app::Task::new(
+                                                    app::MsgIn::External(msg),
+                                                    None,
+                                                ),
+                                            )?;
                                         }
                                     }
                                     Ok(None) => {}
@@ -411,7 +483,10 @@ impl Runner {
                                         if s.success() {
                                             Ok(())
                                         } else {
-                                            Err(format!("process exited with code {}", &s))
+                                            Err(format!(
+                                                "process exited with code {}",
+                                                &s
+                                            ))
                                         }
                                     })
                                     .unwrap_or_else(|e| Err(e.to_string()));
@@ -419,10 +494,12 @@ impl Runner {
                                 match pipe_reader::read_all(&app.pipe.msg_in) {
                                     Ok(msgs) => {
                                         for msg in msgs {
-                                            app = app.handle_task(app::Task::new(
-                                                app::MsgIn::External(msg),
-                                                None,
-                                            ))?;
+                                            app = app.handle_task(
+                                                app::Task::new(
+                                                    app::MsgIn::External(msg),
+                                                    None,
+                                                ),
+                                            )?;
                                         }
                                     }
                                     Err(err) => {
@@ -438,8 +515,11 @@ impl Runner {
                             }
 
                             CallLua(func) => {
-                                execute!(terminal.backend_mut(), event::DisableMouseCapture)
-                                    .unwrap_or_default();
+                                execute!(
+                                    terminal.backend_mut(),
+                                    event::DisableMouseCapture
+                                )
+                                .unwrap_or_default();
 
                                 event_reader.stop();
 
@@ -451,10 +531,12 @@ impl Runner {
                                 match call_lua(&app, &lua, &func, false) {
                                     Ok(Some(msgs)) => {
                                         for msg in msgs {
-                                            app = app.handle_task(app::Task::new(
-                                                app::MsgIn::External(msg),
-                                                None,
-                                            ))?;
+                                            app = app.handle_task(
+                                                app::Task::new(
+                                                    app::MsgIn::External(msg),
+                                                    None,
+                                                ),
+                                            )?;
                                         }
                                     }
                                     Ok(None) => {}
@@ -477,15 +559,102 @@ impl Runner {
                                             mouse_enabled = true;
                                         }
                                         Err(e) => {
-                                            app = app.log_error(e.to_string())?;
+                                            app =
+                                                app.log_error(e.to_string())?;
                                         }
                                     }
                                 }
                             }
 
+                            LuaEval(code) => {
+                                execute!(
+                                    terminal.backend_mut(),
+                                    event::DisableMouseCapture
+                                )
+                                .unwrap_or_default();
+
+                                event_reader.stop();
+
+                                terminal.clear()?;
+                                terminal.set_cursor(0, 0)?;
+                                term::disable_raw_mode()?;
+                                terminal.show_cursor()?;
+
+                                let res: Result<Option<Vec<ExternalMsg>>> = lua
+                                    .load(&code)
+                                    .eval()
+                                    .and_then(|v| lua.from_value(v))
+                                    .map_err(Error::from);
+
+                                match res {
+                                    Ok(Some(msgs)) => {
+                                        for msg in msgs {
+                                            app = app.handle_task(
+                                                app::Task::new(
+                                                    app::MsgIn::External(msg),
+                                                    None,
+                                                ),
+                                            )?;
+                                        }
+                                    }
+                                    Ok(None) => {}
+                                    Err(err) => {
+                                        app = app.log_error(err.to_string())?;
+                                    }
+                                };
+
+                                terminal.clear()?;
+                                term::enable_raw_mode()?;
+                                terminal.hide_cursor()?;
+                                event_reader.start();
+
+                                if mouse_enabled {
+                                    match execute!(
+                                        terminal.backend_mut(),
+                                        event::EnableMouseCapture
+                                    ) {
+                                        Ok(_) => {
+                                            mouse_enabled = true;
+                                        }
+                                        Err(e) => {
+                                            app =
+                                                app.log_error(e.to_string())?;
+                                        }
+                                    }
+                                }
+                            }
+
+                            LuaEvalSilently(code) => {
+                                let res: Result<Option<Vec<ExternalMsg>>> = lua
+                                    .load(&code)
+                                    .eval()
+                                    .and_then(|v| lua.from_value(v))
+                                    .map_err(Error::from);
+
+                                match res {
+                                    Ok(Some(msgs)) => {
+                                        for msg in msgs {
+                                            app = app.handle_task(
+                                                app::Task::new(
+                                                    app::MsgIn::External(msg),
+                                                    None,
+                                                ),
+                                            )?;
+                                        }
+                                    }
+                                    Ok(None) => {}
+                                    Err(err) => {
+                                        app = app.log_error(err.to_string())?;
+                                    }
+                                };
+                            }
+
                             Call(cmd) => {
-                                execute!(terminal.backend_mut(), event::DisableMouseCapture)
-                                    .unwrap_or_default();
+                                execute!(
+                                    terminal.backend_mut(),
+                                    event::DisableMouseCapture
+                                )
+                                .unwrap_or_default();
 
                                 event_reader.stop();
 
@@ -500,7 +669,10 @@ impl Runner {
                                         if s.success() {
                                             Ok(())
                                         } else {
-                                            Err(format!("process exited with code {}", &s))
+                                            Err(format!(
+                                                "process exited with code {}",
+                                                &s
+                                            ))
                                         }
                                     })
                                     .unwrap_or_else(|e| Err(e.to_string()));
@@ -509,10 +681,12 @@ impl Runner {
                                 match pipe_reader::read_all(&app.pipe.msg_in) {
                                     Ok(msgs) => {
                                         for msg in msgs {
-                                            app = app.handle_task(app::Task::new(
-                                                app::MsgIn::External(msg),
-                                                None,
-                                            ))?;
+                                            app = app.handle_task(
+                                                app::Task::new(
+                                                    app::MsgIn::External(msg),
+                                                    None,
+                                                ),
+                                            )?;
                                         }
                                     }
                                     Err(err) => {
@@ -540,7 +714,8 @@ impl Runner {
                                             mouse_enabled = true;
                                         }
                                         Err(e) => {
-                                            app = app.log_error(e.to_string())?;
+                                            app =
+                                                app.log_error(e.to_string())?;
                                         }
                                     }
                                 }
@@ -559,7 +734,8 @@ impl Runner {
         terminal.clear()?;
         terminal.set_cursor(0, 0)?;
         execute!(terminal.backend_mut(), term::LeaveAlternateScreen)?;
-        execute!(terminal.backend_mut(), event::DisableMouseCapture).unwrap_or_default();
+        execute!(terminal.backend_mut(), event::DisableMouseCapture)
+            .unwrap_or_default();
         term::disable_raw_mode()?;
         terminal.show_cursor()?;
 
